/*
  Java gfx-test framework

   Copyright (C) 2010, 2011  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.framework;

import java.awt.Graphics2D;
import java.awt.geom.Path2D;



/**
 * This class contains static methods for rendering common shapes (circles,
 * rectangles etc.) onto specified graphics canvas. These methods can be called
 * from gfx. tests.
 *
 * @author Pavel Tisnovsky
 */
public class CommonShapesRenderer
{
    /**
     * Radius of all four corners of rounded rectangle.
     */
    private static final int ROUND_RECT_RADIUS = 80;

    /**
     * Calculate radius of circle/arc/rectangle based on image size.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @return radius
     */
    public static int calculateRadius(TestImage image)
    {
        return Math.min(image.getWidth(), image.getHeight()) / 3;
    }

    /**
     * Draw cross into graphics canvas, to mark shape's end points, curves's
     * control points etc. Size of cross is stored in constant CROSS_SIZE.
     * 
     * @param graphics
     *            graphics context for the image created by test case
     * @param x
     *            x-coordinate of the cross center
     * @param y
     *            y-coordinate of the cross center
     * @param size
     *            size of cross (measured in pixels)
     */
    private static void drawCross(Graphics2D graphics, int x, int y)
    {
        GfxTest.drawCross(graphics, x, y);
    }

    /**
     * Draw cross into graphics canvas, to mark shape's end points, curves's
     * control points etc. Size of cross is stored in constant CROSS_SIZE.
     * 
     * @param graphics
     *            graphics context for the image created by test case
     * @param x
     *            x-coordinate of the cross center
     * @param y
     *            y-coordinate of the cross center
     */
    private static void drawCross(Graphics2D graphics, int x, int y, int crossSize)
    {
        GfxTest.drawCross(graphics, x, y, crossSize);
    }

    /**
     * Draw crosses at interesting points of image.
     * 
     * @param graphics
     *            graphics context for image
     * @param xc
     *            center of image
     * @param yc
     *            center of image
     * @param radius
     *            radius of shape(s)
     */
    private static void drawCrossesForBasicShapes(Graphics2D graphics, int xc, int yc, int radius)
    {
        // move in vertical direction
        for (int y = yc - radius; y <= yc + radius; y += radius)
        {
            // move in horizontal direction
            for (int x = xc - radius; x <= xc + radius; x += radius)
            {
                drawCross(graphics, x, y);
            }
        }
    }

    /**
     * Draw crosses at interesting points of image.
     * 
     * @param graphics
     *            graphics context for image
     * @param width
     *            width of the shape
     * @param height
     *            height of the shape
     */
    private static void drawCrossesForPolygonShape(Graphics2D graphics, int width, int height)
    {
        final int offset = GfxTest.OFFSET;

        // draw all marks
        drawCross(graphics, offset, height >> 1, GfxTest.CROSS_SIZE);
        drawCross(graphics, width >> 1, offset, GfxTest.CROSS_SIZE);
        drawCross(graphics, width >> 1, height - offset, GfxTest.CROSS_SIZE);
        drawCross(graphics, width - offset, height >> 1, GfxTest.CROSS_SIZE);
    }

    /**
     * Draw crosses at interesting points of image.
     * 
     * @param graphics
     *            graphics context for image
     * @param width
     *            width of the shape
     * @param height
     *            height of the shape
     */
    private static void drawCrossesForClosedPath(Graphics2D graphics, int width, int height)
    {
        final int offset = GfxTest.OFFSET;

        // draw all marks
        drawCross(graphics, offset, height - offset);
        drawCross(graphics, offset, offset);
        drawCross(graphics, width >> 1, offset);
        drawCross(graphics, width - offset, offset);
        drawCross(graphics, width - offset, height - offset);
        drawCross(graphics, width >> 1, offset);
        drawCross(graphics, width >> 1, height - offset);
    }

    /**
     * Construct closed path with given size. The path contains quadratic and
     * also cubic Bezier curve segment.
     * 
     * @param width
     *            width of the shape
     * @param height
     *            height of the shape
     * @return constructed closed path
     */
    private static Path2D constructClosedPath(int width, int height)
    {
        final int offset = GfxTest.OFFSET;

        // construct closed path
        Path2D path = new Path2D.Float();
        path.moveTo(offset, height - offset);

        // path contains quadratic Bezier curve segment
        path.quadTo(offset, offset, width >> 1, offset);
        path.quadTo(width - offset, offset, width - offset, height - offset);

        // and also cubic Bezier curve segment
        path.curveTo(width >> 1, offset, width >> 1, height - offset, offset, height - offset);
        path.closePath();
        return path;
    }

    /**
     * Draw circle onto the graphics canvas.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void drawCircle(TestImage image, Graphics2D graphics)
    {
        // calculate center of the image
        int xc = image.getCenterX();
        int yc = image.getCenterY();
        
        // calculate radius of circle
        int radius = calculateRadius(image);
        
        // draw the circle
        graphics.drawOval(xc - radius, yc - radius, radius << 1, radius << 1);

        // draw crosses at interesting points of image
        drawCrossesForBasicShapes(graphics, xc, yc, radius);
    }

    /**
     * Draw rectangle onto the graphics canvas.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void drawRectangle(TestImage image, Graphics2D graphics)
    {
        // calculate center of the image
        int xc = image.getCenterX();
        int yc = image.getCenterY();
        
        // calculate size of the rectangle
        int size = calculateRadius(image);
        
        // draw the rectangle
        graphics.drawRect(xc - size, yc - size, size << 1, size << 1);

        // draw crosses at interesting points of image
        drawCrossesForBasicShapes(graphics, xc, yc, size);
    }

    /**
     * Draw rounded rectangle onto the graphics canvas.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void drawRoundedRectangle(TestImage image, Graphics2D graphics)
    {
        // calculate center of the image
        int xc = image.getCenterX();
        int yc = image.getCenterY();

        // calculate size of rounded rectangle
        int size = calculateRadius(image);

        // draw the filled round round rectangle
        graphics.drawRoundRect(xc - size, yc - size, size << 1, size << 1, ROUND_RECT_RADIUS, ROUND_RECT_RADIUS);

        // draw crosses at interesting points of image
        drawCrossesForBasicShapes(graphics, xc, yc, size);
    }

    /**
     * Draw filled arc onto the graphics canvas.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void drawArc(TestImage image, Graphics2D graphics)
    {
        // calculate center of the image
        int xc = image.getCenterX();
        int yc = image.getCenterY();
        
        // calculate radius of circle
        int radius = calculateRadius(image);
        
        // draw the empty arc
        graphics.drawArc(xc - radius, yc - radius, radius << 1, radius << 1, 0, 135 + 180);

        // draw crosses at interesting points of image
        drawCrossesForBasicShapes(graphics, xc, yc, radius);
    }

    /**
     * Draw polygon onto the graphics canvas.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void drawPolygon(TestImage image, Graphics2D graphics)
    {
        // calculate image size
        int w = image.getWidth();
        int h = image.getHeight();

        // polygon vertexes
        int xPoints[] = new int[] { GfxTest.OFFSET, w >> 1, w >> 1, w - GfxTest.OFFSET };
        int yPoints[] = new int[] { h >> 1, GfxTest.OFFSET, h - GfxTest.OFFSET, h >> 1 };

        // draw the polygon
        graphics.drawPolygon(xPoints, yPoints, xPoints.length);

        // draw crosses at interesting points of image
        drawCrossesForPolygonShape(graphics, w, h);
    }

    /**
     * Draw filled closed path onto the graphics canvas.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void drawClosedPath(TestImage image, Graphics2D graphics)
    {
        // calculate image size
        int w = image.getWidth();
        int h = image.getHeight();

        // construct closed path with given size
        Path2D path = constructClosedPath(w, h);

        // draw the closed path
        graphics.draw(path);

        // draw crosses at interesting points of image
        drawCrossesForClosedPath(graphics, w, h);
    }

    /**
     * Draw filled circle onto the graphics canvas.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void drawFilledCircle(TestImage image, Graphics2D graphics)
    {
        // calculate center of the image
        int xc = image.getCenterX();
        int yc = image.getCenterY();
        
        // calculate radius of circle
        int radius = calculateRadius(image);
        
        // draw the filled circle
        graphics.fillOval(xc - radius, yc - radius, radius << 1, radius << 1);

        // draw crosses at interesting points of image
        drawCrossesForBasicShapes(graphics, xc, yc, radius);
    }

    /**
     * Draw filled arc onto the graphics canvas.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void drawFilledArc(TestImage image, Graphics2D graphics)
    {
        // calculate center of the image
        int xc = image.getCenterX();
        int yc = image.getCenterY();
        
        // calculate radius of circle
        int radius = calculateRadius(image);
        
        // draw the filled arc
        graphics.fillArc(xc - radius, yc - radius, radius << 1, radius << 1, 0, 135 + 180);

        // draw crosses at interesting points of image
        drawCrossesForBasicShapes(graphics, xc, yc, radius);
    }

    /**
     * Draw filled rectangle onto the graphics canvas.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void drawFilledRectangle(TestImage image, Graphics2D graphics)
    {
        // calculate center of the image
        int xc = image.getCenterX();
        int yc = image.getCenterY();

        // calculate radius of circle
        int radius = calculateRadius(image);

        // draw the filled rectangle
        graphics.fillRect(xc - radius, yc - radius, radius << 1, radius << 1);

        // draw crosses at interesting points of image
        drawCrossesForBasicShapes(graphics, xc, yc, radius);
    }

    /**
     * Draw filled rounded rectangle onto the graphics canvas.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void drawFilledRoundedRectangle(TestImage image, Graphics2D graphics)
    {
        // calculate center of the image
        int xc = image.getCenterX();
        int yc = image.getCenterY();

        // calculate size of rounded rectangle
        int size = calculateRadius(image);

        // draw the filled round round rectangle
        graphics.fillRoundRect(xc - size, yc - size, size << 1, size << 1, ROUND_RECT_RADIUS, ROUND_RECT_RADIUS);

        // draw crosses at interesting points of image
        drawCrossesForBasicShapes(graphics, xc, yc, size);
    }

    /**
     * Draw filled polygon onto the graphics canvas.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void drawFilledPolygon(TestImage image, Graphics2D graphics)
    {
        // calculate image size
        int w = image.getWidth();
        int h = image.getHeight();

        // polygon vertexes
        int xPoints[] = new int[] { GfxTest.OFFSET, w >> 1, w >> 1, w - GfxTest.OFFSET };
        int yPoints[] = new int[] { h >> 1, GfxTest.OFFSET, h - GfxTest.OFFSET, h >> 1 };

        // draw the polygon
        graphics.fillPolygon(xPoints, yPoints, xPoints.length);

        // draw crosses at interesting points of image
        drawCrossesForPolygonShape(graphics, w, h);
    }

    /**
     * Draw filled closed path onto the graphics canvas.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void drawFilledClosedPath(TestImage image, Graphics2D graphics)
    {
        // calculate image size
        int w = image.getWidth();
        int h = image.getHeight();

        // construct closed path with given size
        Path2D path = constructClosedPath(w, h);

        // draw the closed path
        graphics.fill(path);

        // draw crosses at interesting points of image
        drawCrossesForClosedPath(graphics, w, h);
    }

}
